Package: cards
Title: Analysis Results Data
Version: 0.8.1
Authors@R: c(
    person("Daniel D.", "Sjoberg", , "danield.sjoberg@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-0862-2018")),
    person("Becca", "Krouse", , "becca.z.krouse@gsk.com", role = "aut"),
    person("Emily", "de la Rua", , "emily.de_la_rua@contractors.roche.com", role = "aut",
           comment = c(ORCID = "0009-0000-8738-5561")),
    person("Davide", "Garolini", , "dgarolini@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-1445-1369")),
    person("Malan", "Bosman", , "malanbos@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-3020-195X")),
    person("F. Hoffmann-La Roche AG", role = c("cph", "fnd")),
    person("Alanah", "Jonas", , "alanah.x.jonas@gsk.com", role = "aut"),
    person("GlaxoSmithKline Research & Development Limited", role = "cph")
  )
Description: Construct CDISC (Clinical Data Interchange Standards
    Consortium) compliant Analysis Results Data objects. These objects are
    used and re-used to construct summary tables, visualizations, and
    written reports. The package also exports utilities for working with
    these objects and creating new Analysis Results Data objects.
License: Apache License 2.0
URL: https://github.com/insightsengineering/cards,
        https://insightsengineering.github.io/cards/
BugReports: https://github.com/insightsengineering/cards/issues
Depends: R (>= 4.1)
Imports: cli (>= 3.6.5), dplyr (>= 1.1.4), glue (>= 1.8.0), lifecycle
        (>= 1.0.4), rlang (>= 1.1.6), tidyr (>= 1.3.1), tidyselect (>=
        1.2.1)
Suggests: testthat (>= 3.2.3), withr (>= 3.0.0)
Config/Needs/coverage: hms
Config/Needs/website: rmarkdown, jsonlite, yaml, gtsummary, tfrmt,
        cardx, gt, fontawesome, insightsengineering/crane,
        insightsengineering/nesttemplate
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Config/testthat/parallel: true
Encoding: UTF-8
Language: en-US
LazyData: true
NeedsCompilation: no
Packaged: 2026-07-06 05:20:52 UTC; sjobergd
Author: Daniel D. Sjoberg [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0862-2018>),
  Becca Krouse [aut],
  Emily de la Rua [aut] (ORCID: <https://orcid.org/0009-0000-8738-5561>),
  Davide Garolini [aut] (ORCID: <https://orcid.org/0000-0002-1445-1369>),
  Malan Bosman [aut] (ORCID: <https://orcid.org/0000-0002-3020-195X>),
  F. Hoffmann-La Roche AG [cph, fnd],
  Alanah Jonas [aut],
  GlaxoSmithKline Research & Development Limited [cph]
Maintainer: Daniel D. Sjoberg <danield.sjoberg@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-06 15:00:15 UTC
Built: R 4.6.1; ; 2026-07-06 19:21:13 UTC; unix
