# Generated by roxygen2: do not edit by hand

S3method(ard_attributes,data.frame)
S3method(ard_attributes,default)
S3method(ard_categorical,data.frame)
S3method(ard_complex,data.frame)
S3method(ard_continuous,data.frame)
S3method(ard_dichotomous,data.frame)
S3method(ard_hierarchical,data.frame)
S3method(ard_hierarchical_count,data.frame)
S3method(ard_missing,data.frame)
S3method(ard_mvsummary,data.frame)
S3method(ard_summary,data.frame)
S3method(ard_tabulate,data.frame)
S3method(ard_tabulate_value,data.frame)
S3method(ard_total_n,data.frame)
S3method(fill_formula_selectors,data.frame)
S3method(print,card)
S3method(print,compare_ard)
S3method(process_formula_selectors,data.frame)
S3method(process_selectors,data.frame)
export("%>%")
export(add_calculated_row)
export(alias_as_fmt_fn)
export(alias_as_fmt_fun)
export(all_ard_group_n)
export(all_ard_groups)
export(all_ard_variables)
export(all_missing_columns)
export(all_of)
export(any_of)
export(apply_fmt_fn)
export(apply_fmt_fun)
export(ard_attributes)
export(ard_categorical)
export(ard_complex)
export(ard_continuous)
export(ard_dichotomous)
export(ard_formals)
export(ard_hierarchical)
export(ard_hierarchical_count)
export(ard_identity)
export(ard_missing)
export(ard_mvsummary)
export(ard_pairwise)
export(ard_stack)
export(ard_stack_hierarchical)
export(ard_stack_hierarchical_count)
export(ard_strata)
export(ard_summary)
export(ard_tabulate)
export(ard_tabulate_rows)
export(ard_tabulate_value)
export(ard_total_n)
export(as_card)
export(as_cards_fn)
export(as_nested_list)
export(bind_ard)
export(captured_condition_as_error)
export(captured_condition_as_message)
export(cards_select)
export(check_ard_equal)
export(check_ard_structure)
export(check_list_elements)
export(compare_ard)
export(compute_formula_selector)
export(contains)
export(continuous_summary_fns)
export(default_stat_labels)
export(ends_with)
export(eval_capture_conditions)
export(everything)
export(fill_formula_selectors)
export(filter_ard_hierarchical)
export(get_ard_statistics)
export(get_cards_fn_stat_names)
export(is_ard_equal)
export(is_cards_fn)
export(label_round)
export(last_col)
export(matches)
export(maximum_variable_value)
export(mock_attributes)
export(mock_categorical)
export(mock_continuous)
export(mock_dichotomous)
export(mock_missing)
export(mock_total_n)
export(nest_for_ard)
export(num_range)
export(one_of)
export(print_ard_conditions)
export(process_formula_selectors)
export(process_selectors)
export(rename_ard_columns)
export(rename_ard_groups_reverse)
export(rename_ard_groups_shift)
export(replace_null_statistic)
export(round5)
export(shuffle_ard)
export(sort_ard_hierarchical)
export(starts_with)
export(tidy_ard_column_order)
export(tidy_ard_row_order)
export(tidy_as_ard)
export(unlist_ard_columns)
export(update_ard_fmt_fn)
export(update_ard_fmt_fun)
export(update_ard_stat_label)
export(vars)
export(where)
import(rlang)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,contains)
importFrom(dplyr,ends_with)
importFrom(dplyr,everything)
importFrom(dplyr,last_col)
importFrom(dplyr,matches)
importFrom(dplyr,num_range)
importFrom(dplyr,one_of)
importFrom(dplyr,starts_with)
importFrom(dplyr,vars)
importFrom(dplyr,where)
importFrom(lifecycle,deprecated)
