# dplyr ------------------------------------------------------------------------
#' @export
#' @importFrom dplyr %>%
dplyr::`%>%`

#' @importFrom dplyr starts_with
#' @export
dplyr::starts_with

#' @importFrom dplyr ends_with
#' @export
dplyr::ends_with

#' @importFrom dplyr contains
#' @export
dplyr::contains

#' @importFrom dplyr matches
#' @export
dplyr::matches

#' @importFrom dplyr num_range
#' @export
dplyr::num_range

#' @importFrom dplyr all_of
#' @export
dplyr::all_of

#' @importFrom dplyr any_of
#' @export
dplyr::any_of

#' @importFrom dplyr everything
#' @export
dplyr::everything

#' @importFrom dplyr where
#' @export
dplyr::where

#' @importFrom dplyr last_col
#' @export
dplyr::last_col

#' @importFrom dplyr one_of
#' @export
dplyr::one_of

#' @importFrom dplyr vars
#' @export
dplyr::vars
