#' Example ADaM Data
#'
#' Data frame imported from the [CDISC SDTM/ADaM Pilot Project](https://github.com/cdisc-org/sdtm-adam-pilot-project)
#' @name adam
#' @keywords datasets
"ADSL"

#' @rdname adam
"ADAE"

#' @rdname adam
"ADTTE"

#' @rdname adam
"ADLB"
