Done
----
* 3.7, 3.8, 3.9
* flit
* clean up Makefile
* clean up tox.ini
* typer
* black
* pre-commit


Really short term TODO items
----------------------------

* clean up typer usage (maybe)
* validate all docs
* revisit all tests


Long Term TODO items
--------------------

-   Create a REST API and multi-user capability so that multiple people
    can connect to one database.
-   Create alternative front ends. For example:
    -   remote CLI
    -   Flask
    -   Other web frameworks
-   Deploy some version to a cloud based solution.
-   Continue to explore all of the troubling difficult points in blog
    posts and podcast episodes.
-   Change the history.md document into something more sane. Anyone know
    a good way to keep a change log that is relevant but not difficult
    to maintain?
-   Keep it a reasonably simple project that is a good example of
    whatever the Python community generally agrees is ok practices.
