# ard_tabulate_rows() works

    Code
      ard_tabulate_rows(ADSL, by = TRTA)
    Message
      {cards} data frame: 3 x 11
    Output
        group1 group1_level      variable variable_level stat_name stat_label stat
      1   TRTA      Placebo ..row_count..           TRUE         n          n   86
      2   TRTA    Xanomeli… ..row_count..           TRUE         n          n   84
      3   TRTA    Xanomeli… ..row_count..           TRUE         n          n   84
    Message
      i 4 more variables: context, fmt_fun, warning, error

